#include "example.h"

#define OPT_ARG_LIST        "hd:i:"

static void usage( char *pgmName );

/********************************************************
**
** Test Module: GetInfo.c
**
** Purpose:     Get the version of the device driver
**              and the board's firmware version.
**
**
*********************************************************/

int main (int argc, char *argv[])
{
    TPRO_BoardObj   *hnd = NULL;
    char            firmwareVer [TPRO_FIRMWARE_LENGTH + 1];
    char            drvVersion [TPRO_DRV_VERSION_LENGTH + 2];
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[MAX_DEVICE_PATH_LENGTH];
    char            *errMsg;
    char            *pgmname = argv[0];
    int             tidx;
    char            *tstName;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf_s( fullDevName, MAX_DEVICE_PATH_LENGTH, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TPRO_open(&hnd, fullDevName)) != TPRO_SUCCESS )
    {
        /* 
         * if errors, close it
         */
        printf("%s: Could not open <%s> : rc <%d>\n", 
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* 
     * Display the TPRO/TSAT board options 
     */
    (void)printf("<%s> PCI SubsysID: 0x<%04x> PCI devID 0x<%4x>\n",
                    fullDevName, hnd->options, hnd->devid );


    /*
     * perform two tests to get both driver and firmware version
     */
    for ( tidx = 0 ; tidx < 2 ; tidx++ )
    {
        /*
         * perform the appropriate get operation
         */
        if (tidx == 0)
        {
            tstName = "DRV Version";
            rc = TPRO_getDriver(hnd, drvVersion);
        }
        else
        {
            tstName = "FRMWARE Version";
            rc = TPRO_getFirmware(hnd, firmwareVer);
        }

        if ( rc != TPRO_SUCCESS )
        {
            switch (rc)
            {
                case TPRO_HANDLE_ERR:
                    errMsg = "TPRO_HANDLE_ERR";
                    break;
                case TPRO_COMM_ERR:
                    errMsg = "TPRO_COMM_ERR";
                    break;
                default:
                    errMsg = "Unknown error";
                    break;
            }
            (void) printf("%s: <%s>  %s rc: <%d>.\n", 
                        tstName, fullDevName, errMsg, rc);

            ret = 1;
            break;
        }
        else
        {
            /*
             * success:  output driver or firmware version info
             */
            if (tidx == 0 )
            {
                drvVersion[TPRO_DRV_VERSION_LENGTH]='\0';
                (void)printf("<%s> %s: <%s>\n", fullDevName, tstName,  drvVersion);
            }
            else
            {
                (void)printf("<%s> %s: <%s>\n", fullDevName, tstName, firmwareVer );
            }
        }
    }


    /*
     * close the handle to the board
     */
    if ( (rc = TPRO_close(hnd)) != TPRO_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n", 
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n\n", DEFAULT_DEV);
}

